#!/usr/bin/env python
from ds485_const import *
from ds485.dsuid import *
from util import *

def createMessage(cmd, data):
    tot = 3
    if data:
        tot += len(data)
    r = [ cmd ]
    r.append(tot & 0xff)
    r.append(tot >> 8)
    if data:
        r.extend(data)
    return r

class Container:
    def __init__(self, destId, sourceId, ctype, cflags):
        self.destId = destId
        self.sourceId = sourceId
        self.ctype = ctype
        self.cflags = cflags
        #self.length = len(data) # no + 34
        #self.transId = -1
        #self.data

    def _ser_ (self, transId):
        s = list()
        s.extend(self.destId)
        s.extend(self.sourceId)
        s.append(self.ctype)
        s.extend([0] * 3)
        s.append(self.cflags)
        s.extend([0] * 3)
        s.append(len(self.data))
        s.append(transId)
        s.extend(self.data)
        return s

def createBusChange(dsuid, flag):
    c = Container([0] * 17, dsuid, 0, 0) # ctype / cflag don't apply
    c.data = [flag]
    return createMessage(CMD.DS485D_CMD_BUS_CHANGE, c._ser_(0))

class BusChange:
    flag = -1
    dsuid = -1

def parseBusChange(pkt):
    r = BusChange()
    r.dsuid = intern_DSUID(pkt[3 + 17:3 + 17 + 17])
    r.flag = pkt[-1]
    r.joined = pkt[-1] == 0
    return r

# BusState.DS485_STATE_ACTIVE)
def createQueryBusState(state):
    return createMessage(CMD.DS485D_CMD_QUERY_BUSSTATE, [state])

def createBusState(state):
    c = Container([0] * 17, [0] * 17, 0, 0)
    c.data = [state]
    return createMessage(CMD.DS485D_CMD_BUS_STATE, c._ser_(0))

def createDeviceListQuery():
    # 63 -> query list size
    return createMessage(CMD.DS485D_CMD_QUERY_DEVICES, [ 63 ])

def createDeviceListReply(dsuids):
    assert(type(dsuids) == list)
    tmp = []
    map(tmp.extend, dsuids) # convert list of list -> list
    return createMessage(CMD.DS485D_CMD_QUERY_DEVICES, tmp)

def parseQueryDevices(pkt):
    plen = (pkt[2] << 8) + pkt[1]
    pkt = pkt[3:]
    dsuids = []
    while len(pkt) >= 17:
        dsuid = intern_DSUID(pkt[:17])
        dsuids.append(dsuid)
        pkt = pkt[17:]
    if len(dsuids) * 17 + 3 != plen:
        return -1
    return dsuids

def createPingMessage():
    return createMessage(CMD.DS485D_CMD_PING, None)

def createQueryDsuidReply(dsuid):
    return createMessage(CMD.DS485D_CMD_QUERY_DSUID, dsuid)

def createBabble():
    return createMessage(CMD.DS485D_CMD_RX_PACKET, range(20))

def process_pkt(pkt, log, state):
    if pkt[0] == CMD.DS485D_CMD_QUERY_BUSSTATE:
        log.info("ds485d-%d query_busstate %d", state.port, state.connect)
        return createQueryBusState(state.connect)

    elif pkt[0] == CMD.DS485D_CMD_BUS_STATE:
        log.info("ds485d-%d bus_state %d", state.port, state.connect)
        return createBusState(state.connect)

    elif pkt[0] == CMD.DS485D_CMD_PING:
        log.info("ds485d-%d ping", state.port)
        return createPingMessage()

    elif pkt[0] == CMD.DS485D_CMD_QUERY_DSUID:
        log.info("ds485d-%d query_dsuid %s", state.port, state.dsuid)
        return createQueryDsuidReply(state.dsuid.octets)

    elif pkt[0] == CMD.DS485D_CMD_PACKETFILTER:
        log.info("ds485d-%d packet_filter", state.port)
        return None

    elif pkt[0] == CMD.DS485D_CMD_QUERY_DEVICES:
        log.info("ds485d-%d query devices %r", state.port, state.dsmids)
        return createDeviceListReply([d.octets for d in state.dsmids])

    elif (pkt[0] == CMD.DS485D_CMD_RX_PACKET or
          pkt[0] == CMD.DS485D_CMD_TX_PACKET):
        pass

    else:
        log.info("unhandled message 0x%02x", pkt[0])
        return None

def process_pkt_client(pkt, log, state):
    if pkt[0] == CMD.DS485D_CMD_QUERY_BUSSTATE:
        if pkt[0x3] == BusState.DS485_STATE_ACTIVE:
            state.connected = True
        else:
            state.connected = False
    elif pkt[0] == CMD.DS485D_CMD_BUS_STATE:
        #  offset container->data
        if pkt[0x2f] == BusState.DS485_STATE_ACTIVE:
            state.connected = True
        else:
            state.connected = False
    elif pkt[0] == CMD.DS485D_CMD_BUS_CHANGE:
        bc = parseBusChange(pkt)
        log.info("dsm %s %s", bc.dsuid, "joined" if (bc.flag == 0) else "left")
        log.info("%r", state.dsmids)
        if bc.flag == 1:
            if bc.dsuid not in state.dsmids:
                log.error("unknown dsm left: %s", bc.dsuid)
            else:
                state.dsmids.remove(bc.dsuid)
        else:
            if bc.dsuid in state.dsmids:
                log.error("known dsm joined: %s", bc.dsuid)
            else:
                state.dsmids.append(bc.dsuid)
    elif pkt[0] == CMD.DS485D_CMD_QUERY_DEVICES:
        dsuids = parseQueryDevices(pkt)
        if dsuids == -1:
            log.warn("invalid device list")
            log.warn("pkt: %s",  list2hexstr(pkt))
        else:
            state.dsmids = dsuids
    else:
        log.info("unhandled pkt cmd%d", pkt[0])
    return None


def split_pkt(s):
    while len(s) >= 3:
        _len = s[1] + (s[2] <<  8)
        if _len > len(s):
            break
        yield s[0:_len]
        del s[0:_len]

if __name__ == "__main__":
    s = [ 1, 3, 0, 1, 3, 0, 1, 3, 0, 255]
    for a in split_pkt(s):
        print a

    s = [1, 3, 0]
    for a in split_pkt(s):
        pass
    assert(len(s) == 0)

    createBusChange(range(17), 1)

    pkt = [ 20, 3 * 17 + 3, 0]
    pkt.extend(createDssId())
    pkt.extend(createDsmId())
    pkt.extend(createDsmId())
    ret = parseQueryDevices(pkt)
    print ret
